/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "support.h"
#include "pixmaps.h"
#include "check_file.h"

int poll_log_timeout (gpointer *);
int split(char *, char *, char *, char *, char *, char *,char *);
extern void lookup (gchar *, gchar *);

FILE * open_logfile(char *);

#define VERSION "1.0.3"

GtkWidget *logclist;
GtkWidget *window1;
GtkWidget *window2;

GdkColormap *colormap;
GdkBitmap *clist_mask;

GdkPixmap *pm_info;
GdkPixmap *pm_warning;
GdkPixmap *pm_priority;
GdkPixmap *pm_critic;
GdkPixmap *pm_clear;

GtkObject *status_dock;

static FILE *logfile;

long logreset=0;
int resolve=1;
int polling=0;

int main (int argc, char *argv[])
{

#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	gnome_init ("razorback", VERSION, argc, argv);
	
	window1 = create_window1 ();
	gtk_widget_show (window1);
	logclist = lookup_widget(window1,"clist1");
	
	colormap	= gtk_widget_get_colormap(logclist);
	pm_clear	= gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,&clist_mask,NULL,clear_xpm);
	pm_info		= gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,&clist_mask,NULL,info_xpm);
	pm_warning	= gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,&clist_mask,NULL,warning_xpm);
	pm_priority = gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,&clist_mask,NULL,priority_xpm);
	pm_critic	= gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,&clist_mask,NULL,critic_xpm);

	gtk_timeout_add (1000, (GtkFunction)poll_log_timeout, (gpointer)&logreset);

	gtk_main ();

	// free the pixmaps here..
	g_free(colormap); g_free(clist_mask);
	g_free(pm_clear); g_free(pm_info); g_free(pm_warning); g_free(pm_priority); g_free(pm_critic);

	return 0;
}

int poll_log_timeout (gpointer *fullfile)
{
	char buffer[1024];
	char datetime[512]="",description[1024]="",classification[512]="",priority[512]="",protocol[512]="",details[1024]="";
	
	gchar *logfilename=gnome_config_get_string ("/razorback/Files/logfile=/var/log/snort/alert");	
	
	static long filepointer=-1;
	static int firstrun=1;
	int found=0;
	int row;
	int level;
	
	gchar *data[6];
		
	// If this routine is still in the process of running, return back again.
	// Otherwise, we may interrupt a dns query, which may never return again.
	if(polling) {
		return FALSE;
	} else {
		polling=TRUE;
	}
	
	// Check for new / changed files
	if(checkinode(logfilename) != FILE_OK) {
		if(logfile != (FILE *)NULL) {
			fclose(logfile);
		}

		logfile=open_logfile(logfilename);
		
		if(logfile == (FILE *)NULL) {
			g_free(logfilename);
			polling=FALSE;
			return FALSE;
		}
		filepointer=0;
	}
	
	if((long)*fullfile == (long)1) {
		filepointer=0;
	}

	if(firstrun)
	{
		firstrun=0;

		// Jump to the end of the file
		fseek(logfile,0,SEEK_END);
		// test: start of file instead.
		//fseek(logfile,0,SEEK_SET);
	} else {
		if(fseek(logfile,filepointer,SEEK_SET)) {
			// Could not seek to the predetermined file position.
			// This probably means that the file has been rotated.
			fseek(logfile,0,SEEK_SET);
		}
	}

	found=0;
	while(fgets (buffer, 1024, logfile) != NULL) {
		if(split(buffer,datetime,description,classification,priority,protocol,details)) {
			char resolved[1024]="";
			
			found=1;

			data[0]=g_strdup("");
			data[1]=g_strdup(datetime);
			data[2]=g_strdup(description);
			data[3]=g_strdup(classification);
			data[4]=g_strdup(protocol);
			// If the user wants to resolve the hosts,
			// and this is not a full file reload.
			if(resolve && ((long)*fullfile != (long)1)) {
				lookup(details,resolved);
				data[5]=g_strdup(resolved);
			} else {
				data[5]=g_strdup(details);
			}
			
			row=gtk_clist_append(GTK_CLIST(logclist), data);
			
			level=atoi(priority);
			
			if(level == 1) {
				gtk_clist_set_pixmap(GTK_CLIST(logclist),row,0,pm_info,clist_mask);
			} else if(level == 2) {
				gtk_clist_set_pixmap(GTK_CLIST(logclist),row,0,pm_warning,clist_mask);
			} else if(level == 3) {
				gtk_clist_set_pixmap(GTK_CLIST(logclist),row,0,pm_priority,clist_mask);
			} else if(level == 4) {
				gtk_clist_set_pixmap(GTK_CLIST(logclist),row,0,pm_critic,clist_mask);
			} else {
				gtk_clist_set_pixmap(GTK_CLIST(logclist),row,0,pm_clear,clist_mask);
			}
			
			g_free(data[0]); g_free(data[1]); g_free(data[2]); g_free(data[3]); g_free(data[4]); g_free(data[5]);
			// do I need to free data too?
		}
	}
	if(found) {
		gtk_clist_moveto (GTK_CLIST (logclist), GTK_CLIST (logclist)->rows -1, 0, 1.0, 0.0);
		found=0;
	}

	filepointer=ftell(logfile);
	
	*fullfile=0;
	
	g_free(logfilename);
	
	polling=FALSE;
	
	return TRUE;
}


int split(char *input, char *datetime, char *description, char *classification, char *priority, char *protocol,char *details)
{
	int assigned;

	// Assuming a format of:
	// 05/22-17:02:44.055419  [**] [1:466:1] ICMP L3retriever Ping [**] [Classification: Attempted Information Leak] [Priority: 2] {ICMP} 123.8.241.1 -> 123.8.243.248

	assigned = sscanf(input,"%[^ ]  %*4c %*c%*[:0-9]] %[^[] %*4c %*c%*[^:]%*c %[^]]%*c %*c%*[^:]%*c %[^]]%*c %*c%[^}]%*c %[^\n]",datetime,description,classification,priority,protocol,details);

	if(strlen(description)) {
		// Kill off the last space in the description field
		description[strlen(description)-1]='\0';
	}

	return(assigned);
}

FILE * open_logfile(char *filename) {
	logfile = fopen(filename, "r");
	if(logfile == (FILE *)NULL) {
		return(FALSE);
	}

	return(logfile);
}

